import os
from pysfml._core import CsSound, System

class Sound:
  def __init__(self, path):
    s_path = System.String(os.path.abspath(path))
    self._obj = System.Activator.CreateInstance(CsSound, s_path)

  def set_volume(self, v: float):
    self._obj.SetVolume(v)

  def play(self):
    self._obj.Play()

  def stop(self):
    self._obj.Stop()

  def pause(self):
    self._obj.Pause()


import os
from pysfml._core import CsLabel, System

class Label:
  def __init__(self, font_path: str, size: int):
    s_font_path = System.String(os.path.abspath(font_path))
    u_size = System.UInt32(size)
    self._obj = System.Activator.CreateInstance(CsLabel, s_font_path, u_size)
    
  def set_text(self, value):
    self._obj.SetText(value)
    
  def set_position(self, x: float, y: float):
    self._obj.SetPosition(System.Single(x), System.Single(y))
    
  def draw(self, win):
    self._obj.Draw(win._obj.GetRenderWindow())


from pysfml.window import Window
from pysfml.sprite import Sprite
from pysfml.label  import Label
from pysfml.sound  import Sound

__all__ = ["Window", "Sprite", "Label", "Sound"]

__version__ = "1.1.4"
__author__  = "Neyzi"


import os
from pysfml._core import CsSprite, System

class Sprite:
  def __init__(self, width, height):
    try:
      f_width = System.Single(width)
      f_height = System.Single(height)
      self._x = 0.0
      self._y = 0.0
      self._w = float(width)
      self._h = float(height)
      self._obj = System.Activator.CreateInstance(CsSprite, f_width, f_height)
    except Exception as e:
      raise RuntimeError(f"Failed to create Sprite: {e}")

  def set_position(self, x, y):
    self.x, self.y = x, y
    return self._obj.SetPosition(System.Single(x), System.Single(y))
  
  def set_size(self, w, h):
    self._w = float(w)
    self._h = float(h)
    self._obj.SetSize(System.Single(w), System.Single(h))
  
  def set_fill_color(self, r,g,b,a=255):
    self._obj.SetFillColor(System.Byte(r), System.Byte(g), System.Byte(b), System.Byte(a))

  def set_texture_sprite(self, x, y, w, h):
    self._obj.SetTextureRect(x, y, w, h)

  def load_texture(self, path):
    abs_path = os.path.abspath(path)
    self._obj.LoadTexture(abs_path)

  def draw(self, win):
    win.draw(self)


from pysfml._core import CsWindow, Keyboard, Mouse, System

class Window:
  def __init__(self, width: int, height: int, title: str):
    u_width = System.UInt32(width)
    u_height = System.UInt32(height)
    s_title = System.String(title)
    self._obj = System.Activator.CreateInstance(CsWindow, u_width, u_height, s_title)

  def is_key_pressed(self, key_name: str) -> bool:
    key = getattr(Keyboard.Key, key_name, None)
    if key is None:
      raise ValueError(f"Unknown key: '{key_name}'")
    return self._obj.IsKeyPressed(key)
  
  def is_button_pressed(self, button_name: str) -> bool:
    btn = getattr(Mouse.Button, button_name, None)
    if btn is None:
      raise ValueError(f"Unknown key: '{button_name}'")
    return self._obj.IsMouseButtonPressed(btn)
  
  def get_mouse_position(self):
    return self._obj.GetMousePosition()

  def is_open(self):
    return self._obj.isOpen()
  
  def dispatch_events(self):
    self._obj.DispatchEvents()

  def get_delta_time(self) -> float:
    return self._obj.GetDeltaTime()

  def clear(self):
    self._obj.Clear()
  def close(self):
    self._obj.Close()

  def display(self):
    self._obj.Display()

  def draw(self, sprite_wrapper):
    rect = getattr(sprite_wrapper._obj, 'rect', None)
    if rect is not None:
      self._obj.Draw(rect)


import os
import sys
import ctypes
import pythonnet
pythonnet.load("coreclr")
import clr
import System

_dll_dir = os.getcwd()

_lib_dir = os.path.join(os.path.dirname(__file__), "libs")

for _dir in (_lib_dir, _dll_dir):
    if os.path.exists(_dir):
        os.add_dll_directory(_dir)
        os.environ["PATH"] = _dir + os.pathsep + os.environ.get("PATH", "")
        if _dir not in sys.path:
            sys.path.insert(0, _dir)

try:
    clr.AddReference("SFML.System")
    clr.AddReference("SFML.Window")
    clr.AddReference("SFML.Graphics")
    clr.AddReference("SFML.Audio")
    _assembly = clr.AddReference("pysfmllib")
except Exception as e:
    raise ImportError(
        f"pysfml: failed to load SFML assemblies from '{_dll_dir}'.\n"
        f"Make sure csfml-*.dll and sfml-*-3.dll "
        f"are in the same folder as your script.\n"
        f"Original error: {e}"
    )

try:
    from SFML.Window import Keyboard, Mouse
except Exception as e:
    raise ImportError(f"pysfml: could not import SFML.Window enums: {e}")

def _get(name: str):
    t = _assembly.GetType(name)
    if t is None:
        raise ImportError(
            f"pysfml: type '{name}' not found in pysfmllib.dll. "
        )
    return t

CsWindow = _get("pysfmllib.Window")
CsSprite = _get("pysfmllib.Sprite")
CsLabel  = _get("pysfmllib.Label")
CsSound  = _get("pysfmllib.Sound")